@@ -54,8 +54,9 @@ class LTI:
 
         Parameters
         ----------
-        strict: bool (default = False)
-            If strict is True, make sure that timebase is not None
+        strict: bool, optional
+            If strict is True, make sure that timebase is not None.  Default 
+            is False. 
         """
 
         # If no timebase is given, answer depends on strict flag
@@ -73,8 +74,9 @@ class LTI:
         ----------
         sys : LTI system
             System to be checked
-        strict: bool (default = False)
-            If strict is True, make sure that timebase is not None
+        strict: bool, optional
+            If strict is True, make sure that timebase is not None.  Default 
+            is False. 
         """
         # If no timebase is given, answer depends on strict flag
         if self.dt is None:
@@ -176,6 +178,28 @@ def timebaseEqual(sys1, sys2):
     else:
         return sys1.dt == sys2.dt
 
+# Find a common timebase between two or more systems
+def _find_timebase(sys1, *sysn):
+    """Find the common timebase between systems, otherwise return False"""
+
+    # Create a list of systems to check
+    syslist = [sys1]
+    syslist.append(*sysn)
+
+    # Look for a common timebase
+    dt = None
+
+    for sys in syslist:
+        # Make sure time bases are consistent
+        if (dt is None and sys.dt is not None) or \
+           (dt is True and isdiscrete(sys)):
+            # Timebase was not specified; set to match this system
+            dt = sys.dt
+        elif dt != sys.dt:
+            return False
+    return dt
+
+
 # Check to see if a system is a discrete time system
 def isdtime(sys, strict=False):
     """
@@ -198,6 +222,15 @@ def isdtime(sys, strict=False):
     if isinstance(sys, LTI):
         return sys.isdtime(strict)
 
+    # Check to see if object has a dt object
+    if hasattr(sys, 'dt'):
+        # If no timebase is given, answer depends on strict flag
+        if sys.dt == None:
+            return True if not strict else False
+
+        # Look for dt > 0 (also works if dt = True)
+        return sys.dt > 0
+
     # Got passed something we don't recognize
     return False
 
@@ -223,6 +256,13 @@ def isctime(sys, strict=False):
     if isinstance(sys, LTI):
         return sys.isctime(strict)
 
+    # Check to see if object has a dt object
+    if hasattr(sys, 'dt'):
+        # If no timebase is given, answer depends on strict flag
+        if sys.dt is None:
+            return True if not strict else False
+        return sys.dt == 0
+
     # Got passed something we don't recognize
     return False
 
