@@ -53,8 +53,8 @@ $Id$
 
 import math
 import numpy as np
-from numpy import all, angle, any, array, asarray, concatenate, cos, delete, \
-    dot, empty, exp, eye, isinf, matrix, ones, pad, shape, sin, zeros, squeeze
+from numpy import any, array, asarray, concatenate, cos, delete, \
+    dot, empty, exp, eye, isinf, ones, pad, sin, zeros, squeeze
 from numpy.random import rand, randn
 from numpy.linalg import solve, eigvals, matrix_rank
 from numpy.linalg.linalg import LinAlgError
@@ -62,32 +62,62 @@ import scipy as sp
 from scipy.signal import lti, cont2discrete
 from warnings import warn
 from .lti import LTI, timebase, timebaseEqual, isdtime
-from .xferfcn import _convert_to_transfer_function
+from . import config
 from copy import deepcopy
 
 __all__ = ['StateSpace', 'ss', 'rss', 'drss', 'tf2ss', 'ssdata']
 
 
-def _matrix(a):
-    """Wrapper around numpy.matrix that reshapes empty matrices to be 0x0
+# Define module default parameter values
+_statesp_defaults = {
+    'statesp.use_numpy_matrix':True,
+}
+
+
+def _ssmatrix(data, axis=1):
+    """Convert argument to a (possibly empty) state space matrix.
 
     Parameters
     ----------
-    a: sequence passed to numpy.matrix
+    data : array, list, or string
+        Input data defining the contents of the 2D array
+    axis : 0 or 1
+        If input data is 1D, which axis to use for return object.  The default
+        is 1, corresponding to a row matrix.
 
     Returns
     -------
-    am: result of numpy.matrix(a), except if a is empty, am will be 0x0.
+    arr : 2D array, with shape (0, 0) if a is empty
 
-    numpy.matrix([]) has size 1x0; for empty StateSpace objects, we
-    need 0x0 matrices, so use this instead of numpy.matrix in this
-    module.
     """
-    from numpy import matrix
-    am = matrix(a, dtype=float)
-    if (1, 0) == am.shape:
-        am.shape = (0, 0)
-    return am
+    # Convert the data into an array or matrix, as configured
+    # If data is passed as a string, use (deprecated?) matrix constructor
+    if config.defaults['statesp.use_numpy_matrix'] or isinstance(data, str):
+        arr = np.matrix(data, dtype=float)
+    else:
+        arr = np.array(data, dtype=float)
+    ndim = arr.ndim
+    shape = arr.shape
+
+    # Change the shape of the array into a 2D array
+    if (ndim > 2):
+        raise ValueError("state-space matrix must be 2-dimensional")
+
+    elif (ndim == 2 and shape == (1, 0)) or \
+         (ndim == 1 and shape == (0, )):
+        # Passed an empty matrix or empty vector; change shape to (0, 0)
+        shape = (0, 0)
+
+    elif ndim == 1:
+        # Passed a row or column vector
+        shape = (1, shape[0]) if axis == 1 else (shape[0], 1)
+
+    elif ndim == 0:
+        # Passed a constant; turn into a matrix
+        shape = (1, 1)
+
+    #  Create the actual object used to store the result
+    return arr.reshape(shape)
 
 
 class StateSpace(LTI):
@@ -104,18 +134,28 @@ class StateSpace(LTI):
     where u is the input, y is the output, and x is the state.
 
     The main data members are the A, B, C, and D matrices.  The class also
-    keeps track of the number of states (i.e., the size of A).
+    keeps track of the number of states (i.e., the size of A).  The data
+    format used to store state space matrices is set using the value of
+    `config.defaults['use_numpy_matrix']`.  If True (default), the state space
+    elements are stored as `numpy.matrix` objects; otherwise they are
+    `numpy.ndarray` objects.  The :func:`~control.use_numpy_matrix` function
+    can be used to set the storage type.
 
-    Discrete-time state space system are implemented by using the 'dt' instance
-    variable and setting it to the sampling period.  If 'dt' is not None,
-    then it must match whenever two state space systems are combined.
+    Discrete-time state space system are implemented by using the 'dt'
+    instance variable and setting it to the sampling period.  If 'dt' is not
+    None, then it must match whenever two state space systems are combined.
     Setting dt = 0 specifies a continuous system, while leaving dt = None
     means the system timebase is not specified.  If 'dt' is set to True, the
-    system will be treated as a discrete time system with unspecified
-    sampling time.
+    system will be treated as a discrete time system with unspecified sampling
+    time.
+
     """
 
-    def __init__(self, *args):
+    # Allow ndarray * StateSpace to give StateSpace._rmul_() priority
+    __array_priority__ = 11     # override ndarray and matrix types
+
+
+    def __init__(self, *args, **kw):
         """
         StateSpace(A, B, C, D[, dt])
 
@@ -128,7 +168,6 @@ class StateSpace(LTI):
         call StateSpace(sys), where sys is a StateSpace object.
 
         """
-
         if len(args) == 4:
             # The user provided A, B, C, and D matrices.
             (A, B, C, D) = args
@@ -152,7 +191,17 @@ class StateSpace(LTI):
         else:
             raise ValueError("Needs 1 or 4 arguments; received %i." % len(args))
 
-        A, B, C, D = [_matrix(M) for M in (A, B, C, D)]
+        # Process keyword arguments
+        remove_useless = kw.get('remove_useless', True)
+
+        # Convert all matrices to standard form
+        A = _ssmatrix(A)
+        B = _ssmatrix(B, axis=0)
+        C = _ssmatrix(C, axis=1)
+        if np.isscalar(D) and D == 0 and B.shape[1] > 0 and C.shape[0] > 0:
+            # If D is a scalar zero, broadcast it to the proper size
+            D = np.zeros((C.shape[0], B.shape[1]))
+        D = _ssmatrix(D)
 
         # TODO: use super here?
         LTI.__init__(self, inputs=D.shape[1], outputs=D.shape[0], dt=dt)
@@ -183,7 +232,7 @@ class StateSpace(LTI):
             raise ValueError("C and D must have the same number of rows.")
 
         # Check for states that don't do anything, and remove them.
-        self._remove_useless_states()
+        if remove_useless: self._remove_useless_states()
 
     def _remove_useless_states(self):
         """Check for states that don't do anything, and remove them.
@@ -194,12 +243,15 @@ class StateSpace(LTI):
 
         """
 
-        # Search for useless states and get the indices of these states
-        # as an array.
+        # Search for useless states and get indices of these states.
+        #
+        # Note: shape from np.where depends on whether we are storing state
+        # space objects as np.matrix or np.array.  Code below will work
+        # correctly in either case.
         ax1_A = np.where(~self.A.any(axis=1))[0]
         ax1_B = np.where(~self.B.any(axis=1))[0]
-        ax0_A = np.where(~self.A.any(axis=0))[1]
-        ax0_C = np.where(~self.C.any(axis=0))[1]
+        ax0_A = np.where(~self.A.any(axis=0))[-1]
+        ax0_C = np.where(~self.C.any(axis=0))[-1]
         useless_1 = np.intersect1d(ax1_A, ax1_B, assume_unique=True)
         useless_2 = np.intersect1d(ax0_A, ax0_C, assume_unique=True)
         useless = np.union1d(useless_1, useless_2)
@@ -324,12 +376,14 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
 
             # Concatenate the various arrays
             A = concatenate(
-                (concatenate((other.A, zeros((other.A.shape[0], self.A.shape[1]))),
-                 axis=1),
-                concatenate((self.B * other.C, self.A), axis=1)), axis=0)
-            B = concatenate((other.B, self.B * other.D), axis=0)
-            C = concatenate((self.D * other.C, self.C),axis=1)
-            D = self.D * other.D
+                (concatenate((other.A,
+                              zeros((other.A.shape[0], self.A.shape[1]))),
+                             axis=1),
+                 concatenate((np.dot(self.B, other.C), self.A), axis=1)),
+                axis=0)
+            B = concatenate((other.B, np.dot(self.B, other.D)), axis=0)
+            C = concatenate((np.dot(self.D, other.C), self.C),axis=1)
+            D = np.dot(self.D, other.D)
 
         return StateSpace(A, B, C, D, dt)
 
@@ -353,9 +407,9 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
 
         # try to treat this as a matrix
         try:
-            X = _matrix(other)
-            C = X * self.C
-            D = X * self.D
+            X = _ssmatrix(other)
+            C = np.dot(X, self.C)
+            D = np.dot(X, self.D)
             return StateSpace(self.A, self.B, C, D, self.dt)
 
         except Exception as e:
@@ -404,14 +458,13 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
 
         Returns a matrix of values evaluated at complex variable s.
         """
-        resp = self.C * solve(s * eye(self.states) - self.A,
-                              self.B) + self.D
+        resp = np.dot(self.C, solve(s * eye(self.states) - self.A,
+                                    self.B)) + self.D
         return array(resp)
 
     # Method for generating the frequency response of the system
     def freqresp(self, omega):
-        """
-        Evaluate the system's transfer func. at a list of freqs, omega.
+        """Evaluate the system's transfer func. at a list of freqs, omega.
 
         mag, phase, omega = self.freqresp(omega)
 
@@ -424,22 +477,25 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
 
              G(exp(j*omega*dt)) = mag*exp(j*phase).
 
-        Inputs
-        ------
-        omega: A list of frequencies in radians/sec at which the system
-            should be evaluated. The list can be either a python list
-            or a numpy array and will be sorted before evaluation.
+        Parameters
+        ----------
+        omega : array
+            A list of frequencies in radians/sec at which the system should be
+            evaluated. The list can be either a python list or a numpy array
+            and will be sorted before evaluation.
 
         Returns
         -------
-        mag: The magnitude (absolute value, not dB or log10) of the system
+        mag : float
+            The magnitude (absolute value, not dB or log10) of the system
             frequency response.
 
-        phase: The wrapped phase in radians of the system frequency
-            response.
+        phase : float
+            The wrapped phase in radians of the system frequency response.
 
-        omega: The list of sorted frequencies at which the response
-            was evaluated.
+        omega : array
+            The list of sorted frequencies at which the response was
+            evaluated.
 
         """
 
@@ -577,7 +633,7 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         C2 = other.C
         D2 = other.D
 
-        F = eye(self.inputs) - sign * D2 * D1
+        F = eye(self.inputs) - sign * np.dot(D2, D1)
         if matrix_rank(F) != self.inputs:
             raise ValueError("I - sign * D2 * D1 is singular to working precision.")
 
@@ -590,15 +646,20 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         E_D2 = E_D2_C2[:, :other.inputs]
         E_C2 = E_D2_C2[:, other.inputs:]
 
-        T1 = eye(self.outputs) + sign * D1 * E_D2
-        T2 = eye(self.inputs) + sign * E_D2 * D1
+        T1 = eye(self.outputs) + sign * np.dot(D1, E_D2)
+        T2 = eye(self.inputs) + sign * np.dot(E_D2, D1)
 
-        A = concatenate((concatenate((A1 + sign * B1 * E_D2 * C1, sign * B1 * E_C2), axis=1),
-                         concatenate((B2 * T1 * C1, A2 + sign * B2 * D1 * E_C2), axis=1)),
-                        axis=0)
-        B = concatenate((B1 * T2, B2 * D1 * T2), axis=0)
-        C = concatenate((T1 * C1, sign * D1 * E_C2), axis=1)
-        D = D1 * T2
+        A = concatenate(
+            (concatenate(
+                (A1 + sign * np.dot(np.dot(B1, E_D2), C1),
+                 sign * np.dot(B1, E_C2)), axis=1),
+             concatenate(
+                 (np.dot(B2, np.dot(T1, C1)),
+                  A2 + sign * np.dot(np.dot(B2, D1), E_C2)), axis=1)),
+            axis=0)
+        B = concatenate((np.dot(B1, T2), np.dot(np.dot(B2, D1), T2)), axis=0)
+        C = concatenate((np.dot(T1, C1), sign * np.dot(D1, E_C2)), axis=1)
+        D = np.dot(D1, T2)
 
         return StateSpace(A, B, C, D, dt)
 
@@ -614,11 +675,11 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
 
         Parameters
         ----------
-        other: LTI
+        other : LTI
             The lower LTI system
-        ny: int, optional
+        ny : int, optional
             Dimension of (plant) measurement output.
-        nu: int, optional
+        nu : int, optional
             Dimension of (plant) control input.
 
         """
@@ -666,7 +727,7 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         F = np.block([[np.eye(ny), -D22], [-Dbar11, np.eye(nu)]])
         if matrix_rank(F) != ny + nu:
             raise ValueError("lft not well-posed to working precision.")
-        
+
         # solve for the resulting ss by solving for [y, u] using [x,
         # xbar] and [w1, w2].
         TH = np.linalg.solve(F, np.block(
@@ -681,7 +742,7 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         H12 = TH[:ny, self.states + other.states + self.inputs - nu:]
         H21 = TH[ny:, self.states + other.states: self.states + other.states + self.inputs - nu]
         H22 = TH[ny:, self.states + other.states + self.inputs - nu:]
-        
+
         Ares = np.block([
             [A + B2.dot(T21), B2.dot(T22)],
             [Bbar1.dot(T11), Abar + Bbar1.dot(T12)]
@@ -741,7 +802,7 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         for i in range(self.outputs):
             for j in range(self.inputs):
                 out[i][j] = lti(asarray(self.A), asarray(self.B[:, j]),
-                                asarray(self.C[i, :]), asarray(self.D[i, j]))
+                                asarray(self.C[i, :]), self.D[i, j])
 
         return out
 
@@ -793,19 +854,21 @@ but B has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         method :  {"gbt", "bilinear", "euler", "backward_diff", "zoh"}
             Which method to use:
 
-               * gbt: generalized bilinear transformation
-               * bilinear: Tustin's approximation ("gbt" with alpha=0.5)
-               * euler: Euler (or forward differencing) method ("gbt" with alpha=0)
-               * backward_diff: Backwards differencing ("gbt" with alpha=1.0)
-               * zoh: zero-order hold (default)
+            * gbt: generalized bilinear transformation
+            * bilinear: Tustin's approximation ("gbt" with alpha=0.5)
+            * euler: Euler (or forward differencing) method ("gbt" with
+              alpha=0)
+            * backward_diff: Backwards differencing ("gbt" with alpha=1.0)
+            * zoh: zero-order hold (default)
 
         alpha : float within [0, 1]
             The generalized bilinear transformation weighting parameter, which
-            should only be specified with method="gbt", and is ignored otherwise
+            should only be specified with method="gbt", and is ignored
+            otherwise
 
         Returns
         -------
-        sysd : StateSpace system
+        sysd : StateSpace
             Discrete time system, with sampling rate Ts
 
         Notes
@@ -939,7 +1002,7 @@ cannot take keywords.")
 
     # If this is a matrix, try to create a constant feedthrough
     try:
-        D = _matrix(sys)
+        D = _ssmatrix(sys)
         return StateSpace([], [], [], D)
     except Exception as e:
         print("Failure to assume argument is matrix-like in" \
@@ -1079,18 +1142,18 @@ def _mimo2siso(sys, input, output, warn_conversion=False):
 
     Parameters
     ----------
-    sys: StateSpace
+    sys : StateSpace
         Linear (MIMO) system that should be converted.
-    input: int
+    input : int
         Index of the input that will become the SISO system's only input.
-    output: int
+    output : int
         Index of the output that will become the SISO system's only output.
-    warn_conversion: bool
-        If True: print a warning message when sys is a MIMO system.
-        Warn that a conversion will take place.
+    warn_conversion : bool, optional
+        If `True`, print a message when sys is a MIMO system,
+        warning that a conversion will take place.  Default is False.
 
     Returns
-    sys: StateSpace
+    sys : StateSpace
         The converted (SISO) system.
     """
     if not (isinstance(input, int) and isinstance(output, int)):
@@ -1341,16 +1404,16 @@ def rss(states=1, outputs=1, inputs=1):
 
     Parameters
     ----------
-    states: integer
+    states : integer
         Number of state variables
-    inputs: integer
+    inputs : integer
         Number of system inputs
-    outputs: integer
+    outputs : integer
         Number of system outputs
 
     Returns
     -------
-    sys: StateSpace
+    sys : StateSpace
         The randomly created linear system
 
     Raises
@@ -1379,16 +1442,16 @@ def drss(states=1, outputs=1, inputs=1):
 
     Parameters
     ----------
-    states: integer
+    states : integer
         Number of state variables
-    inputs: integer
+    inputs : integer
         Number of system inputs
-    outputs: integer
+    outputs : integer
         Number of system outputs
 
     Returns
     -------
-    sys: StateSpace
+    sys : StateSpace
         The randomly created linear system
 
     Raises
@@ -1417,7 +1480,7 @@ def ssdata(sys):
 
     Parameters
     ----------
-    sys: LTI (StateSpace, or TransferFunction)
+    sys : LTI (StateSpace, or TransferFunction)
         LTI system whose data will be returned
 
     Returns
