@@ -63,6 +63,29 @@ class TestStateSpace(unittest.TestCase):
         D623 = np.zeros((3, 2))
         self.sys623 = StateSpace(A623, B623, C623, D623)
 
+    def test_D_broadcast(self):
+        """Test broadcast of D=0 to the right shape"""
+        # Giving D as a scalar 0 should broadcast to the right shape
+        sys = StateSpace(self.sys623.A, self.sys623.B, self.sys623.C, 0)
+        np.testing.assert_array_equal(self.sys623.D, sys.D)
+
+        # Giving D as a matrix of the wrong size should generate an error
+        with self.assertRaises(ValueError):
+            sys = StateSpace(sys.A, sys.B, sys.C, np.array([[0]]))
+
+        # Make sure that empty systems still work
+        sys = StateSpace([], [], [], 1)
+        np.testing.assert_array_equal(sys.D, [[1]])
+
+        sys = StateSpace([], [], [], [[0]])
+        np.testing.assert_array_equal(sys.D, [[0]])
+
+        sys = StateSpace([], [], [], [0])
+        np.testing.assert_array_equal(sys.D, [[0]])
+
+        sys = StateSpace([], [], [], 0)
+        np.testing.assert_array_equal(sys.D, [[0]])
+
     def test_pole(self):
         """Evaluate the poles of a MIMO system."""
 
@@ -568,6 +591,26 @@ class TestDrss(unittest.TestCase):
         np.testing.assert_array_equal(np.array([]),
                                       StateSpace([], [], [], [[1]]).pole())
 
+    def test_copy_constructor(self):
+        # Create a set of matrices for a simple linear system
+        A = np.array([[-1]])
+        B = np.array([[1]])
+        C = np.array([[1]])
+        D = np.array([[0]])
+
+        # Create the first linear system and a copy
+        linsys = StateSpace(A, B, C, D)
+        cpysys = StateSpace(linsys)
+
+        # Change the original A matrix
+        A[0, 0] = -2
+        np.testing.assert_array_equal(linsys.A, [[-1]]) # original value
+        np.testing.assert_array_equal(cpysys.A, [[-1]]) # original value
+
+        # Change the A matrix for the original system
+        linsys.A[0, 0] = -3
+        np.testing.assert_array_equal(cpysys.A, [[-1]]) # original value
+
 
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(TestStateSpace)
