@@ -1,11 +1,25 @@
-# timeresp.py - time-domain simulation routes
-"""Time domain simulation.
+# timeresp.py - time-domain simulation routines
+#
+# This file contains a collection of functions that calculate time
+# responses for linear systems.
 
-This file contains a collection of functions that calculate time
-responses for linear systems.
+"""The :mod:`~control.timeresp` module contains a collection of
+functions that are used to compute time-domain simulations of LTI
+systems.
 
-See doc/conventions.rst#time-series-conventions_ for more information
-on how time series data are represented.
+Arguments to time-domain simulations include a time vector, an input
+vector (when needed), and an initial condition vector.  The most
+general function for simulating LTI systems the
+:func:`forced_response` function, which has the form::
+
+    t, y = forced_response(sys, T, U, X0)
+
+where `T` is a vector of times at which the response should be
+evaluated, `U` is a vector of inputs (one for each time point) and
+`X0` is the initial condition for the system.
+
+See :ref:`time-series-convention` for more information on how time
+series data are represented.
 
 """
 
@@ -61,6 +75,7 @@ from .lti import isdtime, isctime
 __all__ = ['forced_response', 'step_response', 'step_info', 'initial_response',
            'impulse_response']
 
+
 # Helper function for checking array-like parameters
 def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
                          transpose=False):
@@ -170,7 +185,7 @@ def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
 
 # Forced response of a linear system
 def forced_response(sys, T=None, U=0., X0=0., transpose=False,
-                    interpolate=False):
+                    interpolate=False, squeeze=True):
     """Simulate the output of a linear system.
 
     As a convenience for parameters `U`, `X0`:
@@ -185,29 +200,35 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
     sys: LTI (StateSpace, or TransferFunction)
         LTI system to simulate
 
-    T: array-like
+    T: array-like, optional for discrete LTI `sys`
         Time steps at which the input is defined; values must be evenly spaced.
 
     U: array-like or number, optional
         Input array giving input at each time `T` (default = 0).
 
         If `U` is ``None`` or ``0``, a special algorithm is used. This special
-        algorithm is faster than the general algorithm, which is used otherwise.
+        algorithm is faster than the general algorithm, which is used
+        otherwise.
 
     X0: array-like or number, optional
         Initial condition (default = 0).
 
-    transpose: bool
+    transpose: bool, optional (default=False)
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    interpolate:bool
+    interpolate: bool, optional (default=False)
         If True and system is a discrete time system, the input will
         be interpolated between the given time steps and the output
         will be given at system sampling rate.  Otherwise, only return
         the output at the times given in `T`.  No effect on continuous
         time simulations (default = False).
 
+    squeeze: bool, optional (default=True)
+        If True, remove single-dimensional entries from the shape of
+        the output.  For single output systems, this converts the
+        output response to a 1D array.
+
     Returns
     -------
     T: array
@@ -221,11 +242,21 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
     --------
     step_response, initial_response, impulse_response
 
+    Notes
+    -----
+    For discrete time systems, the input/output response is computed using the
+    :scipy-signal:ref:`scipy.signal.dlsim` function.
+
+    For continuous time systems, the output is computed using the matrix
+    exponential `exp(A t)` and assuming linear interpolation of the inputs
+    between time points.
+
     Examples
     --------
     >>> T, yout, xout = forced_response(sys, T, u, X0)
 
     See :ref:`time-series-convention`.
+
     """
     if not isinstance(sys, LTI):
         raise TypeError('Parameter ``sys``: must be a ``LTI`` object. '
@@ -238,6 +269,12 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
     n_inputs = B.shape[1]
     n_outputs = C.shape[0]
 
+    # Convert inputs to numpy arrays for easier shape checking
+    if U is not None:
+        U = np.asarray(U)
+    if T is not None:
+        T = np.asarray(T)
+
     # Set and/or check time vector in discrete time case
     if isdtime(sys, strict=True):
         if T is None:
@@ -245,13 +282,18 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
                 raise ValueError('Parameters ``T`` and ``U`` can\'t both be'
                                  'zero for discrete-time simulation')
             # Set T to equally spaced samples with same length as U
-            T = np.array(range(len(U))) * (1 if sys.dt == True else sys.dt)
+            if U.ndim == 1:
+                n_steps = U.shape[0]
+            else:
+                n_steps = U.shape[1]
+            T = np.array(range(n_steps)) * (1 if sys.dt is True else sys.dt)
         else:
             # Make sure the input vector and time vector have same length
             # TODO: allow interpolation of the input vector
-            if len(U) != len(T):
-                ValueError('Pamameter ``T`` must have same length as'
-                           'input vector ``U``')
+            if (U.ndim == 1 and U.shape[0] != T.shape[0]) or \
+                    (U.ndim > 1 and U.shape[1] != T.shape[0]):
+                ValueError('Pamameter ``T`` must have same elements as'
+                           ' the number of columns in input array ``U``')
 
     # Test if T has shape (n,) or (1, n);
     # T must be array-like and values must be increasing.
@@ -264,8 +306,9 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
                              transpose=transpose)
     dt = T[1] - T[0]
     if not np.allclose(T[1:] - T[:-1], dt):
-        raise ValueError('Parameter ``T``: time values must be equally spaced.')
-    n_steps = len(T)            # number of simulation steps
+        raise ValueError("Parameter ``T``: time values must be "
+                         "equally spaced.")
+    n_steps = T.shape[0]            # number of simulation steps
 
     # create X0 if not given, test if X0 has correct shape
     X0 = _check_convert_array(X0, [(n_states,), (n_states, 1)],
@@ -310,7 +353,7 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
             #   [ u(dt) ] = exp [  0     0    I ] [  u0   ]
             #   [u1 - u0]       [  0     0    0 ] [u1 - u0]
 
-            M = np.bmat([[A * dt, B * dt, np.zeros((n_states, n_inputs))],
+            M = np.block([[A * dt, B * dt, np.zeros((n_states, n_inputs))],
                          [np.zeros((n_inputs, n_states + n_inputs)),
                           np.identity(n_inputs)],
                          [np.zeros((n_inputs, n_states + 2 * n_inputs))]])
@@ -323,55 +366,61 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
                 xout[:, i] = (dot(Ad, xout[:, i-1]) + dot(Bd0, U[:, i-1]) +
                               dot(Bd1, U[:, i]))
             yout = dot(C, xout) + dot(D, U)
-
         tout = T
-        yout = squeeze(yout)
-        xout = squeeze(xout)
 
     else:
         # Discrete type system => use SciPy signal processing toolbox
-        if (sys.dt != True):
+        if sys.dt is not True:
             # Make sure that the time increment is a multiple of sampling time
 
             # First make sure that time increment is bigger than sampling time
-            if dt < sys.dt:
+            # (with allowance for small precision errors)
+            if dt < sys.dt and not np.isclose(dt, sys.dt):
                 raise ValueError("Time steps ``T`` must match sampling time")
 
             # Now check to make sure it is a multiple (with check against
             # sys.dt because floating point mod can have small errors
             elif not (np.isclose(dt % sys.dt, 0) or
                       np.isclose(dt % sys.dt, sys.dt)):
-                raise ValueError("Time steps ``T`` must be multiples of " \
+                raise ValueError("Time steps ``T`` must be multiples of "
                                  "sampling time")
+            sys_dt = sys.dt
+
         else:
-            sys.dt = dt         # For unspecified sampling time, use time incr
+            sys_dt = dt         # For unspecified sampling time, use time incr
 
         # Discrete time simulation using signal processing toolbox
-        dsys = (A, B, C, D, sys.dt)
+        dsys = (A, B, C, D, sys_dt)
 
         # Use signal processing toolbox for the discrete time simulation
-        # Transpose the input to match toolbox convention 
+        # Transpose the input to match toolbox convention
         tout, yout, xout = sp.signal.dlsim(dsys, np.transpose(U), T, X0)
 
         if not interpolate:
             # If dt is different from sys.dt, resample the output
-            inc = int(round(dt / sys.dt))
+            inc = int(round(dt / sys_dt))
             tout = T            # Return exact list of time steps
-            yout = yout[::inc,:]
-            xout = xout[::inc,:]
+            yout = yout[::inc, :]
+            xout = xout[::inc, :]
 
         # Transpose the output and state vectors to match local convention
         xout = sp.transpose(xout)
         yout = sp.transpose(yout)
 
+    # Get rid of unneeded dimensions
+    if squeeze:
+        yout = np.squeeze(yout)
+        xout = np.squeeze(xout)
+
     # See if we need to transpose the data back into MATLAB form
-    if (transpose):
+    if transpose:
         tout = np.transpose(tout)
         yout = np.transpose(yout)
         xout = np.transpose(xout)
 
     return tout, yout, xout
 
+
 def _get_ss_simo(sys, input=None, output=None):
     """Return a SISO or SIMO state-space version of sys
 
@@ -390,8 +439,9 @@ def _get_ss_simo(sys, input=None, output=None):
     else:
         return _mimo2siso(sys_ss, input, output, warn_conversion=warn)
 
+
 def step_response(sys, T=None, X0=0., input=None, output=None,
-                  transpose=False, return_x=False):
+                  transpose=False, return_x=False, squeeze=True):
     # pylint: disable=W0622
     """Step response of a linear system
 
@@ -430,6 +480,11 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
     return_x: bool
         If True, return the state vector (default = False).
 
+    squeeze: bool, optional (default=True)
+        If True, remove single-dimensional entries from the shape of
+        the output.  For single output systems, this converts the
+        output response to a 1D array.
+
     Returns
     -------
     T: array
@@ -445,9 +500,15 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
     --------
     forced_response, initial_response, impulse_response
 
+    Notes
+    -----
+    This function uses the `forced_response` function with the input set to a
+    unit step.
+
     Examples
     --------
     >>> T, yout = step_response(sys, T, X0)
+
     """
     sys = _get_ss_simo(sys, input, output)
     if T is None:
@@ -460,15 +521,17 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
 
     U = np.ones_like(T)
 
-    T, yout, xout = forced_response(sys, T, U, X0,
-                                     transpose=transpose)
+    T, yout, xout = forced_response(sys, T, U, X0, transpose=transpose,
+                                    squeeze=squeeze)
 
     if return_x:
         return T, yout, xout
 
     return T, yout
 
-def step_info(sys, T=None, SettlingTimeThreshold=0.02, RiseTimeLimits=(0.1,0.9)):
+
+def step_info(sys, T=None, SettlingTimeThreshold=0.02,
+              RiseTimeLimits=(0.1, 0.9)):
     '''
     Step response characteristics (Rise time, Settling Time, Peak and others).
 
@@ -561,8 +624,9 @@ def step_info(sys, T=None, SettlingTimeThreshold=0.02, RiseTimeLimits=(0.1,0.9))
 
     return S
 
+
 def initial_response(sys, T=None, X0=0., input=0, output=None,
-                     transpose=False, return_x=False):
+                     transpose=False, return_x=False, squeeze=True):
     # pylint: disable=W0622
     """Initial condition response of a linear system
 
@@ -601,6 +665,11 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
     return_x: bool
         If True, return the state vector (default = False).
 
+    squeeze: bool, optional (default=True)
+        If True, remove single-dimensional entries from the shape of
+        the output.  For single output systems, this converts the
+        output response to a 1D array.
+
     Returns
     -------
     T: array
@@ -614,6 +683,11 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
     --------
     forced_response, impulse_response, step_response
 
+    Notes
+    -----
+    This function uses the `forced_response` function with the input set to
+    zero.
+
     Examples
     --------
     >>> T, yout = initial_response(sys, T, X0)
@@ -631,7 +705,8 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
             T = range(int(np.ceil(max(tvec))))
     U = np.zeros_like(T)
 
-    T, yout, _xout = forced_response(sys, T, U, X0, transpose=transpose)
+    T, yout, _xout = forced_response(sys, T, U, X0, transpose=transpose,
+                                     squeeze=squeeze)
 
     if return_x:
         return T, yout, _xout
@@ -640,7 +715,7 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
 
 
 def impulse_response(sys, T=None, X0=0., input=0, output=None,
-                     transpose=False, return_x=False):
+                     transpose=False, return_x=False, squeeze=True):
     # pylint: disable=W0622
     """Impulse response of a linear system
 
@@ -679,6 +754,11 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
     return_x: bool
         If True, return the state vector (default = False).
 
+    squeeze: bool, optional (default=True)
+        If True, remove single-dimensional entries from the shape of
+        the output.  For single output systems, this converts the
+        output response to a 1D array.
+
     Returns
     -------
     T: array
@@ -692,17 +772,26 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
     --------
     forced_response, initial_response, step_response
 
+    Notes
+    -----
+    This function uses the `forced_response` function to compute the time
+    response. For continuous time systems, the initial condition is altered to
+    account for the initial impulse.
+
     Examples
     --------
     >>> T, yout = impulse_response(sys, T, X0)
+
     """
     sys = _get_ss_simo(sys, input, output)
 
-    # System has direct feedthrough, can't simulate impulse response numerically
+    # System has direct feedthrough, can't simulate impulse response
+    # numerically
     if np.any(sys.D != 0) and isctime(sys):
-        warnings.warn('System has direct feedthrough: ``D != 0``. The infinite '
-                      'impulse at ``t=0`` does not appear in the output. \n'
-                      'Results may be meaningless!')
+        warnings.warn("System has direct feedthrough: ``D != 0``. The "
+                      "infinite impulse at ``t=0`` does not appear in the "
+                      "output.\n"
+                      "Results may be meaningless!")
 
     # create X0 if not given, test if X0 has correct shape.
     # Must be done here because it is used for computations here.
@@ -732,9 +821,8 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
         new_X0 = X0
         U[0] = 1.
 
-    T, yout, _xout = forced_response(
-        sys, T, U, new_X0,
-        transpose=transpose)
+    T, yout, _xout = forced_response(sys, T, U, new_X0, transpose=transpose,
+                                     squeeze=squeeze)
 
     if return_x:
         return T, yout, _xout
