@@ -6,25 +6,25 @@ RMM, 6 Sep 2010
 
 from __future__ import print_function
 
-from scipy import *  # Load the scipy functions
+import numpy as np  # Load the scipy functions
 from control.matlab import *  # Load the controls systems library
 
 # Parameters defining the system
 
 m = 250.0  # system mass
-k = 40.0  # spring constant
-b = 60.0  # damping constant
+k = 40.0   # spring constant
+b = 60.0   # damping constant
 
 # System matrices
-A = matrix([[1, -1, 1.],
-            [1, -k / m, -b / m],
-            [1, 1, 1]])
+A = np.array([[1, -1, 1.],
+             [1, -k/m, -b/m],
+             [1, 1, 1]])
 
-B = matrix([[0],
-            [1 / m],
-            [1]])
+B = np.array([[0],
+             [1/m],
+             [1]])
 
-C = matrix([[1., 0, 1.]])
+C = np.array([[1., 0, 1.]])
 
 sys = ss(A, B, C, 0)
 
